/*
   pravidla:
     1s=1000ms
     Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
     po zhruba 49 dnoch dojde k prekroceniu 32bit limitu
     a citac sa vynuluje
   uzitocne vzorce:
     rychlost  v=s/t [m/s]  draha  s=vt [m]
               v=at [m/s]
     zrychlenie             a=s/t2 [m/s2]
     gravitacne zrychlenie  g=v/t [m/s2], g na Zemi=9.81 m/s2
                            g=s/t2  draha  s=gt2 [m]
*/
// hlavickove subory *******************************************************
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mCamera.h"

// konstruktor *************************************************************
mCamera::mCamera()
{
 scr_w=0.0f; scr_h=0.0f; // pociatocne rozmery okna

 cam_reset(); // reset parametrov kamery

 collision_onoff=1; // detekcia kolizii je zapnuta
 is_collision=0;    // nie je detekovana kolizia
}

// destruktor **************************************************************
mCamera::~mCamera()
{

}

// reset parametrov kamery *************************************************
void mCamera::cam_reset(void)
{
 // naplnenie matice kamery pociatocnymi hodnotami
 for (t_a=0;t_a<16;t_a++) matrix[t_a]=0.0f;
 matrix[0]=1.0f; matrix[5]=1.0f; matrix[10]=1.0f; matrix[15]=1.0f;
 // nastavenie parametrov kamery
 posx=-1.0f; posy=-1.0f; posz=0.0f;
 oldx=posx; oldy=posy; oldz=posz;
 movx=0.0f; movy=0.0f; movz=0.0f;
 angx=0.0f; angy=0.0f; angz=0.0f;
 rotx=0.0f; roty=0.0f; rotz=0.0f;
 targetx=0.0f; targety=0.0f; targetz=0.0f;
 speed=2.0f; slide=2.0f; alt=2.0f;
 fspeed=6.0f; fslide=6.0f; falt=2.0f;
 acceleration=2.0f; gravity=9.81f;
 viewnear=0.1f; viewfar=5000.0f;
 FOV=60.0f;
 JumpPower=0.0f;

 // nastavenie faktorov pre prepocet posunu a rotacie
 movx_factor=-2.0f; movy_factor=-2.0f; movz_factor=-2.0f;
 rotx_factor=10.0f; roty_factor=10.0f; rotz_factor=1.0f;

 // pociatocne zresetovanie posunu
 goForward=0.0f; goBack=0.0f; goUp=0.0f; goDown=0.0f; goRight=0.0f; goLeft=0.0f;
 goJump=0; goFast=0;

 // strelba
 isFire1=0;
 isFire2=0;

 // sposob vypoctu pohybu
 movetype=1;

 // vyska oci AVATARa - y-ova suradnica kamery
 AVATAR_eye=1.8f;
 // sirka AVATARa
 AVATAR_wide=0.4f;

 // reset casu
 cam_resettime();
}

// reset casu **************************************************************
void mCamera::cam_resettime(void)
{
 // reset casu
 actTime=SDL_GetTicks(); oldTime=actTime;
 difTime=actTime-oldTime;
 gravityTime=0.0f;
 ogl_frames=0.0f;

 ogl_fps=0.0f;
}

// reinicializacia kamery **************************************************
void mCamera::cam_reinit(GLfloat w,GLfloat h)
{
 glMatrixMode(GL_PROJECTION); // vyber projekcnej matice
 glLoadIdentity();            // zresetovanie projekcnej matice

 if (h==0.0f) h=1.0f;
 gluPerspective(FOV,w/h,viewnear,viewfar); // nastavenie perspektivneho premietania
 // FOV - zorny uhol kamery, w/h - aspect je pomer vysky a sirky zobrazovanej plochy
 // viewnear - minimalna viditelna vzdialenost, viewfar - maximalna viditelna vzdialenost
 glViewport(0,0,(GLuint) w,(GLuint) h);    // nastavenie viditelnej plochy

 // TRANSFORMUJEME SCENU NIE KAMERU!
 glMatrixMode(GL_MODELVIEW); // vyber modelovej matice
 glLoadIdentity();           // zresetovanie modelovej matice

 scr_w=w; scr_h=h; // ulozenie rozmerov okna
}

// nastavenie perspektivneho premietania ***********************************
void mCamera::cam_SetPerspective(void)
{
 gluPerspective(FOV,scr_w/scr_h,viewnear,viewfar); // nastavenie perspektivneho premietania
}

// rotacia kamery **********************************************************
void mCamera::cam_rotate_xy(GLfloat x,GLfloat y)
{
 // nastavenie rotacie, hodnoty su korigovane pre dosiahnutie realneho vnemu
 rotx=x/rotx_factor; roty=y/roty_factor;
}

// rotacia kamery **********************************************************
void mCamera::cam_rotate_z(GLfloat z)
{
 // nastavenie rotacie, hodnoty su korigovane pre dosiahnutie realneho vnemu
 rotz=z/rotz_factor;
}

// posun kamery ************************************************************
void mCamera::cam_move(void)
{
 // cas... pohyb je zavisly od casu, rotacia nie...
 actTime=SDL_GetTicks();
 difTime=((GLfloat) (actTime-oldTime))/1000.0f;
 oldTime=actTime;

 // INFO: vypis casoveho okamihu uplynuteho medzi vykreslenim dvoch scen
 //fprintf(stderr,"difTime=%f\n",difTime);

 // pocitadlo FPS
 ogl_frames++;
 if (difTime>=0.001f) {
  ogl_fps=ogl_frames/difTime;
  ogl_frames=0.0f;
 }

 // pohyb kamery, ak su stlacene sucasne dve klavesy, avatar sa nehybe
 if (goFast) { // zrychleny pohyb
  movz=(goForward-goBack)*fspeed*difTime; // dopredu a dozadu
  movx=(goRight-goLeft)*fslide*difTime;   // vlavo a vpravo
  movy=(goUp-goDown)*falt*difTime;        // hore a dole
 }
 else {        // normalna rychlost
  movz=(goForward-goBack)*speed*difTime; // dopredu a dozadu
  movx=(goRight-goLeft)*slide*difTime;   // vlavo a vpravo
  movy=(goUp-goDown)*alt*difTime;        // hore a dole
 }

 // pri type pohybu 'chodza' posobi gravitacia
 if (movetype==0) {
  if (gravityTime==0.0f) { // ak je avatar na zemi
   if (goJump) {           // ak bola stlacena klavesa pre skok
    if (ogl_fps!=0.0f) JumpPower=sqrt(1.0f/ogl_fps); // vypocet sily skoku (zavisle od FPS!)
    else JumpPower=sqrt(1.0f);                       // kontrola na FPS=0!
   }
   else JumpPower=0.0f;    // inac nie je skok
  }

  // zvacsovanie sa casu trvania skoku/posobenia gravitacie
  gravityTime+=difTime;

  // vypocet klesania v zavislosti od skoku a gravitacie
  movy=JumpPower-gravity*gravityTime/20.0f;

  // klesanie nemoze byt vacsie ako rozmer avatara, lebo nastanu chyby pri vypocte kolizie
  if (movy<-1.0f) movy=-1.0f;
 }

 // nastavenie posunu, hodnoty su korigovane pre dosiahnutie realneho vnemu
 movx*=movx_factor; movy*=movy_factor; movz*=movz_factor;

 // zrusenie dalsieho mozneho posunu alebo skoku
 goForward=0.0f; goBack=0.0f; goUp=0.0f; goDown=0.0f; goRight=0.0f; goLeft=0.0f;
 goJump=0; goFast=0;
}

// aktualizacia nastaveni kamery *******************************************
void mCamera::cam_update(void)
{
 // ulozenie povodnej polohy
 oldx=posx; oldy=posy; oldz=posz;

 // SPECIALNY PRISTUP K ROTACII A NASLEDNEMU POSUNU KAMERY!
 glMatrixMode(GL_MODELVIEW);
 glLoadMatrixf(matrix); // modelova matica sa nahradi maticou kamery

 switch (movetype) {
  case 0:  // chodza
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,0.0f,1.0f,0.0f);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;
   // ulozenie udajov do matice kamery
   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
   // nastavenie novej polohy - POSUN sa vykona neskor v gl_draw
   posx+=matrix[0]*movx-matrix[2]*movz;
   posy+=movy;
   posz+=matrix[8]*movx-matrix[10]*movz;
   break;
  case 1:  // let
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,matrix[1],matrix[5],matrix[9]);
   glRotatef(rotz,matrix[2],matrix[6],matrix[10]);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;
   // ulozenie udajov do matice kamery
   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
   // nastavenie novej polohy - POSUN sa vykona neskor v gl_draw
   posx+=matrix[0]*movx+matrix[1]*movy-matrix[2]*movz;
   posy+=matrix[4]*movx+matrix[5]*movy-matrix[6]*movz;
   posz+=matrix[8]*movx+matrix[9]*movy-matrix[10]*movz;
   break;
  default: // dalsie typy pohybu...
   break;
 }

 // zrusenie dalsej moznej rotacie
 rotx=0.0f; roty=0.0f; rotz=0.0f;
 // nastavenie smeroveho vektora kamery
 targetx=matrix[2]; targety=matrix[6]; targetz=matrix[10];
}

// zap/vyp detekcie kolizii ************************************************
void mCamera::CollisionOnOff(void)
{
 if (collision_onoff==0) collision_onoff=1; else collision_onoff=0;

 SDL_Delay(200);
}

// ziskanie casoveho prirastku *********************************************
GLfloat mCamera::GetCameraTime(void)
{
 if (difTime>=0.001f) return difTime;
 else return (0.001f);
}
