// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mOGL.h"

#define PI    3.141592653589
#define PI180 0.017453292519

GLUquadricObj *qo; // quadratic objekt

// konstruktor *************************************************************
mOGL::mOGL()
{
 Cnum=0;

 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 StartInfoPos=10.0f; // pozicia vypisu 1. textu pocas startu aplikacie

 t_a=0; t_b=0; t_c=0; // pomocne premenne

 lighting_onoff=1;  // osvetlenie
 help_onoff=0;      // napoveda
 scene_draw_type=0; // standardny pohlad na scenu
 pause_motion=0;    // casovo zavisly pohyb

 light_rot=0.0f;
}

// destruktor **************************************************************
mOGL::~mOGL()
{

}

// vypis info pocas startu aplikacie ***************************************
void mOGL::PrintStartInfo(char *infotext)
{
 ogl_Text.OpenDrawString();
 ogl_Text.DrawString(10.0f,StartInfoPos,infotext);
 ogl_Text.CloseDrawString();

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 // dalsi text bude vypisany na novu poziciu
 StartInfoPos+=20.0f;
 // pauza aby bol text aspon chvilku viditelny
 SDL_Delay(400);
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
bool mOGL::gl_initialize(void)
{
 // nastavenie 8-mich vseobecnych materialov
 for (t_a=0;t_a<8;t_a++) {
  ogl_Material[t_a].amb[0]=0.3f; ogl_Material[t_a].amb[1]=0.3f;
  ogl_Material[t_a].amb[2]=0.3f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.6f; ogl_Material[t_a].dif[1]=0.6f;
  ogl_Material[t_a].dif[2]=0.6f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=0.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 8-mich vseobecnych svetiel
 for (t_a=0;t_a<8;t_a++) {
  ogl_Light[t_a].amb[0]=0.25f; ogl_Light[t_a].amb[1]=0.25f;
  ogl_Light[t_a].amb[2]=0.25f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.5f; ogl_Light[t_a].dif[1]=0.5f;
  ogl_Light[t_a].dif[2]=0.5f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.75f; ogl_Light[t_a].spe[1]=0.75f;
  ogl_Light[t_a].spe[2]=0.75f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=0.0f; ogl_Light[t_a].pos[1]=2.0f;
  ogl_Light[t_a].pos[2]=0.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.0f,0.0f,0.0f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 //glClearStencil(0);                 // stencil zasobnik bude vymazavany nastavenim na 0

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ogl_Material[0].amb_dif);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);
 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 qo=gluNewQuadric();              // pre kvadraticke objekty
 gluQuadricNormals(qo,GL_SMOOTH); // zapnutie generovania normalovych vektorov
 gluQuadricTexture(qo,GL_FALSE);  // vypnutie generovania texturovacich suradnic

 // nastavenie modu generovania koordinatov S
 glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
 // nastavenie modu generovania koordinatov T
 glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);

 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

 // ZAPNUTIE TEXTUROVANIA
 glEnable(GL_TEXTURE_2D);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // vymazanie zasobnika farieb, hlbky a stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // inicializacia vypisovania OpenGL textu
 if (ogl_Text.TextInitialize()) {
  fprintf(stderr,"Chyba inicializacie fontu!\n"); return(1);
 }
 // potvrdenie startu OpenGL
 PrintStartInfo("Nastavenie OpenGL stavoveho stroja...");
 PrintStartInfo("OK");

 // zameriavaci kriz
 PrintStartInfo("Nahratie textury zameriavacieho kriza...");
 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 PrintStartInfo("OK");

 PrintStartInfo("Nacitanie textur...");
 // nahratie textur
 if (ogl_Texture.LoadTexture(0,"data/floor.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 if (ogl_Texture.LoadBumpTexture(1,2,"data/floor_bump.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(3,"data/earth.bmp",TEXTURE_CLAMP|TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 if (ogl_Texture.LoadBumpTexture(4,5,"data/earth_bump.bmp",TEXTURE_CLAMP|TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(6,"data/wall.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 if (ogl_Texture.LoadBumpTexture(7,8,"data/wall_bump.bmp",TEXTURE_MIPMAP|TEXTURE_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textur!\n"); return(1);
 }
 PrintStartInfo("OK");

 // koniec pociatocnych nastaveni, zaciatok interakcie pozorovatela
 PrintStartInfo("START");

 // nastavenie farby a velkosti neskor vypisovaneho textu
 ogl_Text.SetTextColor(1.0f,1.0f,1.0f);
 ogl_Text.SetTextSize(10);

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// pusun texturovacich suradnic invertovanej bump-mapy *********************
void mOGL::doBumps(GLfloat *n,GLfloat *pp,GLfloat *s,GLfloat *t)
{
 // vypocet prvkov vektora bod-svetelny zdroj
 v[0]=lp[0]-pp[0]; v[1]=lp[1]-pp[1]; v[2]=lp[2]-pp[2];
 // normalizacia vektora
 p1=(GLfloat) sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
 if (p1!=0.0f) { v[0]/=p1; v[1]/=p1; v[2]/=p1; }
 // skalarny sucin texturovacich vektorov a vektora bod-svetelny zdroj
 // vysledok sa nasobi tzv. EMBOSS indexom, ktory udava "silu"
 // posunu, prilis velkymi hodnotami dosiahneme nezelany vysledok!
 pp[0]=(s[0]*v[0]+s[1]*v[1]+s[2]*v[2])*0.01f;
 pp[1]=(t[0]*v[0]+t[1]*v[1]+t[2]*v[2])*0.01f;
}

// vykreslenie OpenGL sceny ************************************************
int mOGL::gl_draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glMultMatrixf(ogl_Camera[Cnum].matrix);
 else {
  glRotatef(45.0f,1.0f,0.0f,0.0f);  // zapnutie specialneho pohladu na scenu
  glRotatef(-30.0f,0.0f,1.0f,0.0f);
 }
 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glTranslatef(ogl_Camera[Cnum].posx,ogl_Camera[Cnum].posy,ogl_Camera[Cnum].posz);
 else glTranslatef(-7.5f,-10.0f,-15.0f); // v pripade specialneho pohladu na scenu

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-ogl_Camera[0].posx;
 //ogl_Light[0].pos[1]=-ogl_Camera[0].posy;
 //ogl_Light[0].pos[2]=-ogl_Camera[0].posz;

 // umiestenenie svetla na preddefinovanu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // zmena polohy svetla
 ogl_Light[0].pos[0]=1.0f+1.5f*sin(light_rot*PI180);
 ogl_Light[0].pos[2]=1.5f*cos(light_rot*PI180);
 light_rot+=0.1f; if (light_rot>360.0f) light_rot=0.0f;

 // ziskanie polohy svetelneho zdroja
 lp[0]=ogl_Light[0].pos[0];
 lp[1]=ogl_Light[0].pos[1];
 lp[2]=ogl_Light[0].pos[2];
 lp[3]=1.0f;

 // implementacia bump-mappingu --------------------------------------------
 // 1. krok - vykreslenie objektu s vyuzitim bump-mapy
 glDisable(GL_LIGHTING);

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[1]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-1.0f,0.0f, 1.0f);
  glTexCoord2f(2.0f,0.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f( 3.0f,0.0f, 1.0f);
  glTexCoord2f(2.0f,2.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f( 3.0f,0.0f,-3.0f);
  glTexCoord2f(0.0f,2.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-1.0f,0.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[7]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,0.0f,-3.0f);
  glTexCoord2f(2.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,0.0f,-3.0f);
  glTexCoord2f(2.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,1.0f,-3.0f);
  glTexCoord2f(0.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,1.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[4]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,1.0f,-3.0f);
  glTexCoord2f(1.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,1.0f,-3.0f);
  glTexCoord2f(1.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,2.5f,-3.0f);
  glTexCoord2f(0.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,2.5f,-3.0f);
 glEnd();

 // 2. krok - vykreslenie objektu s vyuzitim invertovnej bump-mapy
 glBlendFunc(GL_ONE,GL_ONE);
 glEnable(GL_BLEND);

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[2]);
 glBegin(GL_QUADS);
  n[0]=0.0f; n[1]=1.0f; n[2]=0.0f; // normala
  s[0]=1.0f; s[1]=0.0f; s[2]=0.0f; // vektor prirastku s-texturovacej suradnice
  t[0]=0.0f; t[1]=0.0f; t[2]=1.0f; // vektor prirastku t-texturovacej suradnice

  pp[0]=-1.0f; pp[1]=0.0f; pp[2]=1.0f; // 1. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],0.0f+pp[1]); glVertex3f(-1.0f,0.0f,1.0f);
  pp[0]=3.0f; pp[1]=0.0f; pp[2]=1.0f; // 2. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(2.0f+pp[0],0.0f+pp[1]); glVertex3f(3.0f,0.0f,1.0f);
  pp[0]=3.0f; pp[1]=0.0f; pp[2]=-3.0f; // 3. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(2.0f+pp[0],2.0f+pp[1]); glVertex3f(3.0f,0.0f,-3.0f);
  pp[0]=-1.0f; pp[1]=0.0f; pp[2]=-3.0f; // 4. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],2.0f+pp[1]); glVertex3f(-1.0f,0.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[8]);
 glBegin(GL_QUADS);
  n[0]=0.0f; n[1]=0.0f; n[2]=1.0f; // normala
  s[0]=1.0f; s[1]=0.0f; s[2]=0.0f; // vektor prirastku s-texturovacej suradnice
  t[0]=0.0f; t[1]=1.0f; t[2]=0.0f; // vektor prirastku t-texturovacej suradnice

  pp[0]=-1.0f; pp[1]=0.0f; pp[2]=-3.0f; // 1. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],0.0f+pp[1]); glVertex3f(-1.0f,0.0f,-3.0f);
  pp[0]=3.0f; pp[1]=0.0f; pp[2]=-3.0f; // 2. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(2.0f+pp[0],0.0f+pp[1]); glVertex3f(3.0f,0.0f,-3.0f);
  pp[0]=3.0f; pp[1]=1.0f; pp[2]=-3.0f; // 3. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(2.0f+pp[0],1.0f+pp[1]); glVertex3f(3.0f,1.0f,-3.0f);
  pp[0]=-1.0f; pp[1]=1.0f; pp[2]=-3.0f; // 4. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],1.0f+pp[1]); glVertex3f(-1.0f,1.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[5]);
 glBegin(GL_QUADS);
  n[0]=0.0f; n[1]=0.0f; n[2]=1.0f; // normala
  s[0]=1.0f; s[1]=0.0f; s[2]=0.0f; // vektor prirastku s-texturovacej suradnice
  t[0]=0.0f; t[1]=1.0f; t[2]=0.0f; // vektor prirastku t-texturovacej suradnice

  pp[0]=-1.0f; pp[1]=1.0f; pp[2]=-3.0f; // 1. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],0.0f+pp[1]); glVertex3f(-1.0f,1.0f,-3.0f);
  pp[0]=3.0f; pp[1]=1.0f; pp[2]=-3.0f; // 2. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(1.0f+pp[0],0.0f+pp[1]); glVertex3f(3.0f,1.0f,-3.0f);
  pp[0]=3.0f; pp[1]=2.5f; pp[2]=-3.0f; // 3. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(1.0f+pp[0],1.0f+pp[1]); glVertex3f(3.0f,2.5f,-3.0f);
  pp[0]=-1.0f; pp[1]=2.5f; pp[2]=-3.0f; // 4. vrchol
  doBumps(n,pp,s,t);
  glTexCoord2f(0.0f+pp[0],1.0f+pp[1]); glVertex3f(-1.0f,2.5f,-3.0f);
 glEnd();

 // 3. krok - vykreslenie objektu s vyuzitim originalnej textury
 glEnable(GL_LIGHTING);

 glBlendFunc(GL_DST_COLOR,GL_SRC_COLOR);

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[0]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-1.0f,0.0f, 1.0f);
  glTexCoord2f(2.0f,0.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f( 3.0f,0.0f, 1.0f);
  glTexCoord2f(2.0f,2.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f( 3.0f,0.0f,-3.0f);
  glTexCoord2f(0.0f,2.0f); glNormal3f(0.0f,1.0f,0.0f); glVertex3f(-1.0f,0.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[6]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,0.0f,-3.0f);
  glTexCoord2f(2.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,0.0f,-3.0f);
  glTexCoord2f(2.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,1.0f,-3.0f);
  glTexCoord2f(0.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,1.0f,-3.0f);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[3]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,1.0f,-3.0f);
  glTexCoord2f(1.0f,0.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,1.0f,-3.0f);
  glTexCoord2f(1.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f( 3.0f,2.5f,-3.0f);
  glTexCoord2f(0.0f,1.0f); glNormal3f(0.0f,0.0f,1.0f); glVertex3f(-1.0f,2.5f,-3.0f);
 glEnd();

 glDisable(GL_BLEND);

 // vykreslenie svetelneho zdroja
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania

 glColor4f(1.0f,1.0f,0.0f,1.0f);
 glTranslatef(ogl_Light[0].pos[0],ogl_Light[0].pos[1],ogl_Light[0].pos[2]);
 gluSphere(qo,0.02f,16,16);
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 glPopAttrib();

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair(ogl_Camera[Cnum].scr_w,ogl_Camera[Cnum].scr_h);

 if (help_onoff) {
  // vypis textu - help
  ogl_Text.OpenDrawString();

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-20.0f, "ESC ukoncenie aplikacie");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-60.0f, "MYS                rotacia v danom smere");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-80.0f, "SIPKY              posun dopredu,dozadu,vlavo,vpravo");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-100.0f,"SHIFT              zrychlenie posunu");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-140.0f,"R     reinicializacia pohladu kamery");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-160.0f,"M     vyplnene trojuholniky/iba hrany trojuholnikov");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-180.0f,"P     zapnutie/vypnutie casovo zavisleho pohybu objektov");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-220.0f,"H     zapnutie/vypnutie zobrazenia helpu");

  ogl_Text.CloseDrawString();
 }
 else {
  // vypis textu - poloha kamery
  ogl_Text.OpenDrawString();

  sprintf(ogl_Text.numberbuf,"x:%6.2f",-ogl_Camera[Cnum].posx);
  ogl_Text.DrawNumber(10.0f,30.0f);
  sprintf(ogl_Text.numberbuf,"y:%6.2f",-ogl_Camera[Cnum].posy);
  ogl_Text.DrawNumber(10.0f,20.0f);
  sprintf(ogl_Text.numberbuf,"z:%6.2f",-ogl_Camera[Cnum].posz);
  ogl_Text.DrawNumber(10.0f,10.0f);

  // vypis textu - pocet obrazkov za sekundu
  sprintf(ogl_Text.numberbuf,"%i fps",(GLuint) ogl_Camera[Cnum].ogl_fps);
  //sprintf(ogl_Text.numberbuf,"%f fps",ogl_Camera[Cnum].GetCameraTime()*1000.0f);
  ogl_Text.DrawNumber(10.0f,ogl_Camera[Cnum].scr_h-20.0f);

  ogl_Text.CloseDrawString();
 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mOGL::gl_iddle(void)
{
 if (pause_motion==0) {

 }

 return(0);
}

// zmena sposobu zobrazenia objektov ***************************************
void mOGL::change_PolygonMode(void)
{
 glGetIntegerv(GL_POLYGON_MODE,&ogl_State[0]);
 // INFO: vypis nacitaneho stavu
 //fprintf(stderr,"ogl_State=%d\n",ogl_State[0]);

 if (ogl_State[0]==GL_LINE) glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 SDL_Delay(200);
}

// zap/vyp casovo zavisleho pohybu objektov ********************************
void mOGL::change_PauseMotion(void)
{
 if (pause_motion==0) pause_motion=1;
 else pause_motion=0;

 SDL_Delay(200);
}

// zap/vyp vypisu helpu ****************************************************
void mOGL::change_HelpOnOff(void)
{
 if (help_onoff==0) help_onoff=1;
 else help_onoff=0;

 SDL_Delay(200);
}
